@@ -1,7 +1,5 @@
 /*
-  Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003,
-  2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014,
-  2015 Free Software Foundation, Inc.
+  Copyright (C) 1995-2020 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -1721,6 +1719,9 @@ shell (int argc, char **argv _GL_UNUSED_PARAMETER)
 void
 user (int argc, char **argv)
 {
+#if !HAVE_DECL_GETPASS
+  extern char *getpass ();
+#endif
   char acct[80];
   int n, aflag = 0;
 
@@ -2070,6 +2071,9 @@ globulize (char *cp)
 void
 account (int argc, char **argv)
 {
+#if !HAVE_DECL_GETPASS
+  extern char *getpass ();
+#endif
   char acct[50], *ap;
 
   if (argc > 1)
