@@ -1,7 +1,5 @@
 /*
-  Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003,
-  2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014,
-  2015 Free Software Foundation, Inc.
+  Copyright (C) 1995-2020 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -286,16 +284,15 @@ main (int argc, char *argv[])
   if (argc > 0)
     {
       char *xargv[5];
+      int i;
 
       if (setjmp (toplevel))
 	exit (EXIT_SUCCESS);
       signal (SIGINT, intr);
       signal (SIGPIPE, lostpeer);
       xargv[0] = program_invocation_name;
-      xargv[1] = argv[0];
-      xargv[2] = argv[1];
-      xargv[3] = argv[2];
-      xargv[4] = NULL;
+      for (i = 0; i < argc && i < 3; i++)
+	xargv[i + 1] = argv[i];
       setpeer (argc + 1, xargv);
     }
   top = setjmp (toplevel) == 0;
