@@ -1,7 +1,5 @@
 /*
-  Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002,
-  2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013
-  Free Software Foundation, Inc.
+  Copyright (C) 1994-2020 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -294,8 +292,8 @@ static struct argp_option options[] = {
   { "timeout", 't', "TIMEOUT", 0,
     "set default idle timeout",
     GRID+1 },
-  { "max-timeout", 'T', NULL, 0,
-    "reset maximum value of timeout allowed",
+  { "max-timeout", 'T', "MAX_TIMEOUT", 0,
+    "set maximum value of timeout allowed",
     GRID+1 },
   { "non-rfc2577", OPT_NONRFC2577, NULL, 0,
     "neglect RFC 2577 by giving info on missing users",
@@ -1206,7 +1204,7 @@ bad:
   /* Return the real value of errno (close may change it) */
   t = errno;
   if (seteuid ((uid_t) cred.uid) != 0)
-    _exit (EXIT_FAILURE);;
+    _exit (EXIT_FAILURE);
   close (s);
   errno = t;
   return NULL;
