@@ -1,6 +1,5 @@
 /* printif.c -- print an interface configuration
-  Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
-  2010, 2011, 2012, 2013, 2014, 2015 Free Software Foundation, Inc.
+  Copyright (C) 2001-2020 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -481,22 +480,29 @@ fh_ifdisplay_query (format_data_t form, int argc, char *argv[])
   int n;
 
 #ifdef SIOCGIFFLAGS
-  int f;
-  int rev;
-  unsigned int uflags = (unsigned short) form->ifr->ifr_flags;
+  /* Request for all, or for a specified interface?  */
+  n = all_option || ifs_cmdline;
+  if (!n)
+    {
+      /* Otherwise, only interfaces in state `UP' are displayed.  */
+      int rev = 0;
+      int f = if_nameztoflag ("UP", &rev);
+
+      n = f && ioctl (form->sfd, SIOCGIFFLAGS, form->ifr) == 0;
+      if (n) {
+	unsigned int uflags = (unsigned short) form->ifr->ifr_flags;
 
 # ifdef ifr_flagshigh
-  uflags |= (unsigned short) form->ifr->ifr_flagshigh << 16;
+	uflags |= (unsigned short) form->ifr->ifr_flagshigh << 16;
 # endif /* ifr_flagshigh */
 
-  n = !(all_option || ifs_cmdline
-	|| ((f = if_nameztoflag ("UP", &rev))
-	    && ioctl (form->sfd, SIOCGIFFLAGS, form->ifr) == 0
-	    && (f & uflags)));
+	n = n && (f & uflags);
+      };
+    }
 #else
-  n = 0;
+  n = 1;	/* Display all of them.  */
 #endif
-  select_arg (form, argc, argv, n);
+  select_arg (form, argc, argv, !n);
 }
 
 void
@@ -712,24 +718,32 @@ fh_brdaddr_query (format_data_t form, int argc, char *argv[])
 # ifdef SIOCGIFFLAGS
   int f;
   int rev;
-  unsigned int uflags = (unsigned short) form->ifr->ifr_flags;
+  unsigned int uflags;
 
-# ifdef ifr_flagshigh
-  uflags |= (unsigned short) form->ifr->ifr_flagshigh << 16;
-# endif /* ifr_flagshigh */
+  f = if_nameztoflag ("BROADCAST", &rev);
 
-  if (0 == (f = if_nameztoflag ("BROADCAST", &rev))
-      || (ioctl (form->sfd, SIOCGIFFLAGS, form->ifr) < 0)
-      || ((f & uflags) == 0))
+  if (f == 0 || (ioctl (form->sfd, SIOCGIFFLAGS, form->ifr) < 0))
     {
       select_arg (form, argc, argv, 1);
       return;
     }
-# endif
+
+  uflags = (unsigned short) form->ifr->ifr_flags;
+#  ifdef ifr_flagshigh
+  uflags |= (unsigned short) form->ifr->ifr_flagshigh << 16;
+#  endif /* ifr_flagshigh */
+
+  if ((f & uflags) == 0)
+    {
+      select_arg (form, argc, argv, 1);
+      return;
+    }
+# endif /* SIOCGIFFLAGS */
+
   if (ioctl (form->sfd, SIOCGIFBRDADDR, form->ifr) >= 0)
     select_arg (form, argc, argv, 0);
   else
-#endif
+#endif /* SIOCGIFBRDADDR */
     select_arg (form, argc, argv, 1);
 }
 
@@ -756,24 +770,32 @@ fh_dstaddr_query (format_data_t form, int argc, char *argv[])
 # ifdef SIOCGIFFLAGS
   int f;
   int rev;
-  unsigned int uflags = (unsigned short) form->ifr->ifr_flags;
+  unsigned int uflags;
 
-#  ifdef ifr_flagshigh
-  uflags |= (unsigned short) form->ifr->ifr_flagshigh << 16;
-#  endif /* ifr_flagshigh */
+  f = if_nameztoflag ("POINTOPOINT", &rev);
 
-  if (0 == (f = if_nameztoflag ("POINTOPOINT", &rev))
-      || (ioctl (form->sfd, SIOCGIFFLAGS, form->ifr) < 0)
-      || ((f & uflags) == 0))
+  if (f == 0 || (ioctl (form->sfd, SIOCGIFFLAGS, form->ifr) < 0))
     {
       select_arg (form, argc, argv, 1);
       return;
     }
-# endif
+
+  uflags = (unsigned short) form->ifr->ifr_flags;
+#  ifdef ifr_flagshigh
+  uflags |= (unsigned short) form->ifr->ifr_flagshigh << 16;
+#  endif /* ifr_flagshigh */
+
+  if ((f & uflags) == 0)
+    {
+      select_arg (form, argc, argv, 1);
+      return;
+    }
+# endif /* SIOCGIFFLAGS */
+
   if (ioctl (form->sfd, SIOCGIFDSTADDR, form->ifr) >= 0)
     select_arg (form, argc, argv, 0);
   else
-#endif
+#endif /* SIOCGIFDSTADDR */
     select_arg (form, argc, argv, 1);
 }
 
