@@ -1,6 +1,5 @@
 /*
-  Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
-  2010, 2011, 2012, 2013, 2014, 2015 Free Software Foundation, Inc.
+  Copyright (C) 2001-2020 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -57,6 +56,12 @@ struct system_ifconfig
 
 /* Output format support.  */
 
+# define _IU_CAT2(a, b) a ## b
+# define _IU_DCL(name, fld)				\
+  {#name, _IU_CAT2(system_fh_, fld) }
+# define _IU_EXTRN(fld)				\
+  extern void _IU_CAT2(system_fh_, fld) (format_data_t, int, char *[]);	\
+
 # define SYSTEM_FORMAT_HANDLER	\
   {"bsd", fh_nothing},		\
   {IU_BSD_TYPE, fh_nothing},	\
@@ -71,7 +76,24 @@ struct system_ifconfig
   {"netmask?", system_fh_netmask_query}, \
   {"netmask", system_fh_netmask},	\
   {"status?", system_fh_status_query}, \
-  {"status", system_fh_status},
+  {"status", system_fh_status}, \
+  {"tunnel?", system_fh_tunnel_query}, \
+  {"tundst", system_fh_tundst}, \
+  {"tunsrc", system_fh_tunsrc}, \
+  {"ifstat?", system_fh_ifstat_query}, \
+  _IU_DCL (collisions, collisions), \
+  _IU_DCL (rxbytes, rx_bytes), \
+  _IU_DCL (rxdropped, rx_dropped), \
+  _IU_DCL (rxerrors, rx_errors), \
+  {"rxfifoerr", system_fh_missing_stat}, \
+  {"rxframeerr", system_fh_missing_stat}, \
+  _IU_DCL (rxpackets, rx_packets), \
+  _IU_DCL (txbytes, tx_bytes), \
+  {"txcarrier", system_fh_missing_stat}, \
+  _IU_DCL (txdropped, tx_dropped), \
+  _IU_DCL (txerrors, tx_errors), \
+  {"txfifoerr", system_fh_missing_stat}, \
+  _IU_DCL (txpackets, tx_packets),
 
 void system_fh_brdaddr_query (format_data_t form, int argc, char *argv[]);
 void system_fh_brdaddr (format_data_t form, int argc, char *argv[]);
@@ -85,5 +107,21 @@ void system_fh_netmask_query (format_data_t form, int argc, char *argv[]);
 void system_fh_netmask (format_data_t form, int argc, char *argv[]);
 void system_fh_status_query (format_data_t form, int argc, char *argv[]);
 void system_fh_status (format_data_t form, int argc, char *argv[]);
+void system_fh_tunnel_query (format_data_t form, int argc, char *argv[]);
+void system_fh_tundst (format_data_t form, int argc, char *argv[]);
+void system_fh_tunsrc (format_data_t form, int argc, char *argv[]);
+
+void system_fh_ifstat_query (format_data_t form, int argc, char *argv[]);
+void system_fh_missing_stat (format_data_t form, int argc, char *argv[]);
+
+_IU_EXTRN (rx_bytes)
+_IU_EXTRN (tx_bytes)
+_IU_EXTRN (rx_dropped)
+_IU_EXTRN (tx_dropped)
+_IU_EXTRN (rx_errors)
+_IU_EXTRN (tx_errors)
+_IU_EXTRN (rx_packets)
+_IU_EXTRN (tx_packets)
+_IU_EXTRN (collisions)
 
 #endif /* IFCONFIG_SYSTEM_BSD_H */
