@@ -1,6 +1,5 @@
 /* linux.c -- Linux specific code for ifconfig
-  Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
-  2010, 2011, 2012, 2013 Free Software Foundation, Inc.
+  Copyright (C) 2001-2020 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -433,7 +432,7 @@ pnd_read ()
 
   while (getline (&buf, &bufsize, fp) > 0)
     {
-      struct pnd_stats *stats = xzalloc (sizeof (*stats));;
+      struct pnd_stats *stats = xzalloc (sizeof (*stats));
       char *p = buf, *q;
       size_t len;
 
@@ -675,6 +674,7 @@ system_fh_txqlen (format_data_t form, int argc, char *argv[])
 const char *system_help = "\
  NAME [ADDR] [broadcast BRDADDR]\
  [pointopoint|dstaddr DSTADDR] [netmask MASK]\
+ [ether|hwaddr|lladdr MACADDR]\
  [metric N] [mtu N] [txqueuelen N] [up|down] [FLAGS]";
 
 void
@@ -763,15 +763,16 @@ system_parse_opt_rest (struct ifconfig **ifp, int argc, char *argv[])
   int i = 0;
   enum
   {
-    EXPECT_INET,
+    EXPECT_AF,
     EXPECT_NOTHING,
     EXPECT_BROADCAST,
     EXPECT_DSTADDR,
     EXPECT_NETMASK,
+    EXPECT_HWADDR,
     EXPECT_MTU,
     EXPECT_METRIC,
     EXPECT_TXQLEN,
-  } expect = EXPECT_INET;
+  } expect = EXPECT_AF;
   int mask, rev;
 
   *ifp = parse_opt_new_ifs (argv[0]);
@@ -792,6 +793,10 @@ system_parse_opt_rest (struct ifconfig **ifp, int argc, char *argv[])
 	  parse_opt_set_netmask (*ifp, argv[i]);
 	  break;
 
+	case EXPECT_HWADDR:
+	  parse_opt_set_hwaddr (*ifp, argv[i]);
+	  break;
+
 	case EXPECT_MTU:
 	  parse_opt_set_mtu (*ifp, argv[i]);
 	  break;
@@ -804,10 +809,15 @@ system_parse_opt_rest (struct ifconfig **ifp, int argc, char *argv[])
 	  system_parse_opt_set_txqlen (*ifp, argv[i]);
 	  break;
 
-	case EXPECT_INET:
+	case EXPECT_AF:
 	  expect = EXPECT_NOTHING;
 	  if (!strcmp (argv[i], "inet"))
 	    continue;
+	  else if (!strcmp (argv[i], "inet6"))
+	    {
+	      error (0, 0, "%s is not a supported address family", argv[i]);
+	      return 0;
+	    }
 	  break;
 
 	case EXPECT_NOTHING:
@@ -823,6 +833,10 @@ system_parse_opt_rest (struct ifconfig **ifp, int argc, char *argv[])
 	expect = EXPECT_DSTADDR;
       else if (!strcmp (argv[i], "netmask"))
 	expect = EXPECT_NETMASK;
+      else if (!strcmp (argv[i], "ether")
+	       || !strcmp(argv[i], "hwaddr")
+	       || !strcmp(argv[i], "lladdr"))
+	expect = EXPECT_HWADDR;
       else if (!strcmp (argv[i], "metric"))
 	expect = EXPECT_METRIC;
       else if (!strcmp (argv[i], "mtu"))
@@ -833,7 +847,8 @@ system_parse_opt_rest (struct ifconfig **ifp, int argc, char *argv[])
 	parse_opt_set_flag (*ifp, IFF_UP | IFF_RUNNING, 0);
       else if (!strcmp (argv[i], "down"))
 	parse_opt_set_flag (*ifp, IFF_UP, 1);
-      else if ((mask = if_nameztoflag (argv[i], &rev)) != 0)
+      else if (((mask = if_nameztoflag (argv[i], &rev))
+		& ~IU_IFF_CANTCHANGE) != 0)
 	parse_opt_set_flag (*ifp, mask, rev);
       else
 	parse_opt_set_address (*ifp, argv[i]);
@@ -853,6 +868,10 @@ system_parse_opt_rest (struct ifconfig **ifp, int argc, char *argv[])
       error (0, 0, "option `netmask' requires an argument");
       break;
 
+    case EXPECT_HWADDR:
+      error (0, 0, "option `hwaddr' requires an argument");
+      break;
+
     case EXPECT_METRIC:
       error (0, 0, "option `metric' requires an argument");
       break;
@@ -865,7 +884,7 @@ system_parse_opt_rest (struct ifconfig **ifp, int argc, char *argv[])
       error (0, 0, "option `txqueuelen' requires an argument");
       break;
 
-    case EXPECT_INET:
+    case EXPECT_AF:
     case EXPECT_NOTHING:
       return 1;
     }
