@@ -1,6 +1,5 @@
 /*
-  Copyright (C) 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015 Free Software
-  Foundation, Inc.
+  Copyright (C) 2008-2020 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -232,7 +231,7 @@ set_name (const hostname_arguments *const args)
   else
     hostname_new = args->hostname_new;
 
-  size = strlen (hostname_new);
+  size = hostname_new ? strlen (hostname_new) : 0;
   if (!size)
     error (EXIT_FAILURE, 0, "Empty hostname");
 
@@ -337,7 +336,10 @@ get_ip_addresses (const char *const host_name)
             }
 
           strcat (addresses, address);
-          strcat (addresses, " ");
+
+	  /* Insert a separating space character.  */
+	  if (ht->h_addr_list[i+1] != NULL)
+	    strcat (addresses, " ");
         }
     }
 
@@ -407,9 +409,11 @@ parse_file (const char *const file_name)
 
       if (buffer[0] != '#')
         {
-	  name = (char *) xmalloc (sizeof (char) * nread);
-	  if (sscanf (buffer, "%s", name)  == 1)
+	  name = (char *) xmalloc (sizeof (char) * (nread + 1));
+	  if (sscanf (buffer, "%s", name) == 1)
 	    break;
+	  free (name);
+	  name = NULL;
         }
     }
   while (feof (file) == 0);
