@@ -1,7 +1,5 @@
 /*
-  Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003,
-  2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013 Free
-  Software Foundation, Inc.
+  Copyright (C) 1995-2020 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -123,12 +121,14 @@
 #if defined KERBEROS || defined SHISHI
 int use_kerberos = 1, doencrypt;
 # ifdef KERBEROS
-char dest_realm_buf[REALM_SZ], *dest_realm = NULL;
+char dest_realm_buf[REALM_SZ];
+const char *dest_realm = NULL;
 CREDENTIALS cred;
 Key_schedule schedule;
 
 # elif defined(SHISHI)
-char dest_realm_buf[REALM_SZ], *dest_realm = NULL;
+char dest_realm_buf[REALM_SZ];
+const char *dest_realm = NULL;
 
 Shishi *handle;
 Shishi_key *key;
