@@ -1,7 +1,5 @@
 /*
-  Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003,
-  2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013 Free
-  Software Foundation, Inc.
+  Copyright (C) 1995-2020 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -221,7 +219,7 @@ main (int argc, char *argv[])
   iu_argp_init ("tftpd", default_program_authors);
   argp_parse (&argp, argc, argv, 0, &index, NULL);
 
-  openlog ("tftpd", LOG_PID, LOG_FTP);
+  openlog ("tftpd", LOG_NDELAY | LOG_PID, LOG_FTP);
 
   if (index < argc)
     {
@@ -416,7 +414,7 @@ main (int argc, char *argv[])
 
 struct formats;
 int validate_access (char **, int);
-void send_file (struct formats *);
+void tftpd_sendfile (struct formats *);
 void recvfile (struct formats *);
 
 struct formats
@@ -428,8 +426,8 @@ struct formats
   int f_convert;
 } formats[] =
   {
-    {"netascii", validate_access, send_file, recvfile, 1},
-    {"octet", validate_access, send_file, recvfile, 0},
+    {"netascii", validate_access, tftpd_sendfile, recvfile, 1},
+    {"octet", validate_access, tftpd_sendfile, recvfile, 0},
     {0, NULL, NULL, NULL, 0}
   };
 
@@ -648,7 +646,7 @@ timer (int sig _GL_UNUSED_PARAMETER)
  * Send the requested file.
  */
 void
-send_file (struct formats *pf)
+tftpd_sendfile (struct formats *pf)
 {
   struct tftphdr *dp, *r_init (void);
   register struct tftphdr *ap;	/* ack packet */
