@@ -1,7 +1,5 @@
 /*
-  Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003,
-  2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013 Free
-  Software Foundation, Inc.
+  Copyright (C) 1995-2020 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -2223,7 +2221,7 @@ telsnd (void)
 	  if ((sc == '\n') || (sc == '\r'))
 	    bol = 1;
 	}
-      else if (sc == escape)
+      else if (escape != _POSIX_VDISABLE && sc == escape)
 	{
 	  /*
 	   * Double escape is a pass through of a single escape character.
