@@ -1,7 +1,5 @@
 /*
-  Copyright (C) 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001,
-  2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012,
-  2013 Free Software Foundation, Inc.
+  Copyright (C) 1993-2020 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -42,6 +40,7 @@ static void parse_debug_level (char *str);
 static void telnetd_setup (int fd);
 static int telnetd_run (void);
 static void print_hostinfo (void);
+static void chld_is_done (int sig);
 
 /* Template command line for invoking login program.  */
 
@@ -51,14 +50,14 @@ char *login_invocation =
    *       `-u' takes the Kerberos principal name
    *       of the authenticating, remote user.
    */
-  PATH_LOGIN " -p -h %h %?T{-t %T} %?u{-u %u}"
+  PATH_LOGIN " -p -h %h %?T{-t %T} -d %L %?u{-u %u}{%U}"
 
 #elif defined SOLARIS
   /* At least for SunOS 5.8.  */
-  PATH_LOGIN " -h %h %?T{%T} %?u{-- %u}"
+  PATH_LOGIN " -h %h %?T{%T} %?u{-- %u}{%U}"
 
 #else /* !SOLARIS */
-  PATH_LOGIN " -p -h %h %?u{-f %u}"
+  PATH_LOGIN " -p -h %h %?u{-f %u}{%U}"
 #endif
   ;
 
@@ -73,6 +72,8 @@ int hostinfo = 1;		/* Print the host-specific information before
 int debug_level[debug_max_mode];	/* Debugging levels */
 int debug_tcp = 0;		/* Should the SO_DEBUG be set? */
 
+int pending_sigchld = 0;	/* Needed to drain pty input.  */
+
 int net;			/* Network connection socket */
 int pty;			/* PTY master descriptor */
 #if defined AUTHENTICATION || defined ENCRYPTION
@@ -501,6 +502,14 @@ telnetd_setup (int fd)
 
   io_setup ();
 
+  /* Before doing anything related to the identity of the client,
+   * scrub the environment variable USER, since it may be set with
+   * an irrelevant user name at this point.  OpenBSD has been known
+   * to offend at this point with their own inetd.  Any demand for
+   * autologin will get attention in getterminaltype().
+   */
+  unsetenv ("USER");
+
   /* get terminal type. */
   uname[0] = 0;
   level = getterminaltype (uname, sizeof (uname));
@@ -527,7 +536,8 @@ telnetd_setup (int fd)
   signal (SIGTTOU, SIG_IGN);
 #endif
 
-  signal (SIGCHLD, cleanup);
+  /* Activate SA_RESTART whenever available.  */
+  setsig (SIGCHLD, chld_is_done);
 }
 
 int
@@ -658,14 +668,23 @@ telnetd_run (void)
       if (FD_ISSET (pty, &ibits))
 	{
 	  /* Something to read from the pty... */
+
+	  /* Observe that pty_read() is masking a few select
+	   * read errors with the return value 0.  Let them
+	   * pass for further manipulation.  Issue reported in
+	   * http://lists.gnu.org/archive/html/bug-inetutils/2015-07/msg00006.html
+	   */
 	  if (pty_read () < 0)
 	    break;
+
+	  /* The first byte is now TIOCPKT data.  Peek at it.  */
 	  c = pty_get_char (1);
+
 #if defined TIOCPKT_IOCTL
 	  if (c & TIOCPKT_IOCTL)
 	    {
 	      pty_get_char (0);
-	      copy_termbuf ();
+	      copy_termbuf ();	/* Pty buffer is now emptied.  */
 	      localstat ();
 	    }
 #endif
@@ -682,16 +701,22 @@ telnetd_run (void)
 	  if (his_state_is_will (TELOPT_LFLOW)
 	      && (c & (TIOCPKT_NOSTOP | TIOCPKT_DOSTOP)))
 	    {
-	      int newflow = c & TIOCPKT_DOSTOP ? 1 : 0;
+	      int newflow = (c & TIOCPKT_DOSTOP) ? 1 : 0;
 	      if (newflow != flowmode)
 		{
-		  net_output_data ("%c%c%c%c%c%c",
-				   IAC, SB, TELOPT_LFLOW,
-				   flowmode ? LFLOW_ON : LFLOW_OFF, IAC, SE);
+		  char data[6];
+
+		  sprintf (data, "%c%c%c%c%c%c",
+			   IAC, SB, TELOPT_LFLOW,
+			   flowmode ? LFLOW_ON : LFLOW_OFF,
+			   IAC, SE);
+		  net_output_datalen (data, sizeof (data));
+		  DEBUG (debug_options, 1,
+			 printsub ('>', data + 2, sizeof (data) - 2));
 		}
-	      pty_get_char (0);
 	    }
 
+	  pty_get_char (0);	/* Discard the TIOCPKT preamble.  */
 	}
 
       while (pty_input_level () > 0)
@@ -718,6 +743,18 @@ telnetd_run (void)
 
       if (FD_ISSET (pty, &obits) && pty_output_level () > 0)
 	ptyflush ();
+
+      /* Attending to the child must come last in the loop,
+       * so as to let pending data be flushed, mainly to the
+       * benefit of the remote and expecting client.
+       */
+      if (pending_sigchld) {
+	/* Check for pending output, independently of OBITS.  */
+	if (net_output_level () > 0)
+	  netflush ();
+
+	cleanup (SIGCHLD);	/* Not returning from this.  */
+      }
     }
 
   cleanup (0);
@@ -754,3 +791,9 @@ print_hostinfo (void)
   pty_input_putback (str, strlen (str));
   free (str);
 }
+
+static void
+chld_is_done (int sig _GL_UNUSED_PARAMETER)
+{
+  pending_sigchld = 1;
+}
