@@ -1,7 +1,5 @@
 /*
-  Copyright (C) 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001,
-  2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012,
-  2013, 2014, 2015 Free Software Foundation, Inc.
+  Copyright (C) 1993-2020 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -372,6 +370,14 @@ pty_input_putback (const char *str, size_t len)
   return 0;
 }
 
+/* pty_read()
+ *
+ * Read errors EWOULDBLOCK, EAGAIN, and EIO are
+ * tweeked into reporting zero bytes input.
+ * In particular, EIO is known to appear when
+ * reading off the master side, before having
+ * an active slave side.
+ */
 int
 pty_read (void)
 {
@@ -1188,9 +1194,21 @@ printsub (int direction, unsigned char *pointer, int length)
 		debug_output_data ("(0x%x)", pointer[i + SLC_FLAGS]);
 	      debug_output_data (" %d;", pointer[i + SLC_VALUE]);
 
+	      /* Protocol enforced duplication of IAC depends on pre
+	       * and post modification of data, hence differs between
+	       * in, out, and no direction, i.e., recursive mode.
+	       * Some systems assign _POSIX_VDISABLE and IAC the same
+	       * value!  Heuristic experiments led to the following.
+	       * Recursive mode needs both steps as written here.
+	       */
 	      if ((pointer[i + SLC_VALUE] == IAC) &&
-		  (pointer[i + SLC_VALUE + 1] == IAC))
+		  (pointer[i + SLC_VALUE + 1] == IAC) &&
+		  (direction != '<'))
 		i++;
+	      if ((pointer[i + SLC_VALUE] == IAC) &&
+		  (pointer[i + SLC_VALUE + 1] == IAC) &&
+		  !direction)
+		i += 2;
 	    }
 
 	  for (; i < length; i++)
@@ -1212,7 +1230,7 @@ printsub (int direction, unsigned char *pointer, int length)
 		      pointer[2] & MODE_SOFT_TAB ? "|SOFT_TAB" : "",
 		      pointer[2] & MODE_LIT_ECHO ? "|LIT_ECHO" : "",
 		      pointer[2] & MODE_ACK ? "|ACK" : "");
-	    debug_output_data ("%s", tbuf[1] ? &tbuf[1] : "0");
+	    debug_output_data ("%s", tbuf[0] ? &tbuf[1] : "0");
 	  }
 
 	  if (pointer[2] & ~(MODE_EDIT | MODE_TRAPSIG | MODE_ACK))
@@ -1564,7 +1582,12 @@ printsub (int direction, unsigned char *pointer, int length)
 	debug_output_data (" %d", pointer[i]);
       break;
     }
-  debug_output_data ("\r\n");
+
+  /* Without direction, we are doing a recursive suboption printing.
+   * Suppress NL, which would otherwise misplace the SE marker.
+   */
+  if (direction)
+    debug_output_data ("\r\n");
 }
 
 /*
